@@ -79,7 +79,7 @@ struct extent_record {
 	struct rb_root backref_tree;
 	struct list_head list;
 	struct cache_extent cache;
-	struct btrfs_disk_key parent_key;
+	struct btrfs_key parent_key;
 	u64 start;
 	u64 max_size;
 	u64 nr;
@@ -185,6 +185,8 @@ struct unaligned_extent_rec_t {
 #define I_ERR_INLINE_RAM_BYTES_WRONG	(1 << 17)
 #define I_ERR_MISMATCH_DIR_HASH		(1 << 18)
 #define I_ERR_INVALID_IMODE		(1 << 19)
+#define I_ERR_INVALID_GEN		(1 << 20)
+#define I_ERR_INVALID_NLINK		(1 << 21)
 
 struct inode_record {
 	struct list_head backrefs;
