@@ -9,12 +9,9 @@ check_prereq btrfs
 
 # redefine the one provided by common
 check_image() {
-	local image
-
-	image=$1
-	run_mayfail $TOP/btrfs check "$image"
+	run_mayfail "$TOP/btrfs" check "$1"
 }
 
-check_all_images $TEST_TOP/fuzz-tests/images
+check_all_images "$TEST_TOP/fuzz-tests/images"
 
 exit 0
